(*based on Q fits err 20120708. plots and fits data for files with 3 columns, first is time stamp just 
before acquiring voltage
point, second column is time just after, third is voltage. Uses simple simnusoid with no damping for "parcel" 
analysis only; verifies "Estimated Variance" in NonLinearModelFit is unweighted ChiSquare, gets mean and errors
on amplitude and frequecy from weighted fit assuming constant error
inferred from noise
Used for official anlysis, data of 20130809*)

Clear[model,paramlist,
      ftext, fmark, pftext, QRawDat, QRawPlt,
      dimraw,freqr, angfreqr, 
      dangfreqr, Qf, beta, ampl, phas, ti, freqr1, angfreqr1, 
      dangfreqr1, Qf1, beta1, ampl1, phas1, fstring, ost] 

(*Need flag*)
ndflg = 1
If[ndflg ==1,
Needs["ErrorBarPlots`"]
]

(*Enter fit function*)

model = ampl Exp[-beta ti] Cos[(2. Pi freqr ti) + phas] + ost
paramlist = {{ampl, ampl0}, {beta, beta0}, {freqr, freqr0}, {phas, phas0}, {ost, ost0}}
(*
model = (ampl Exp[-beta ti]+ampl2 Exp[-beta2 ti]) Cos[(2. Pi freqr ti) + phas] + ost
paramlist = {{ampl, ampl0}, {beta, beta0}, {freqr, freqr0}, {phas, phas0}, {ost, ost0}, {ampl2, ampl20}, {beta2, beta20}}
(*Amplitude2 - First raw voltage point*)
ampl20 = 2.0 10^-5
(*time constant2, s*)
tau20 = 50. (*s*)
beta20 = 1/tau20
*)
(*initial guesses for parameters*)
(*Amplitude - First raw voltage point*)
ampl0 = 1.0 10^-1
(*time constant, s*)
tau0 = 10. (*s*)
beta0 = 1/tau0
(*BEAT frequency - should only be 1-2 Hz*)
freqr0 = 2.0 (*Hz*)
(*Phase*)
phas0 = 0.0
(*Offset*)
ost0 = 0.0

(*Enter file name in quotes*)
fstring = "qdat_2013_08_09_r001x.txt"

Print["Q Data, file = ",fstring]

(*Enter cuts on sets*)
MinQtRawDat = 20.
MaxQtRawDat = 55.
Groundtime  = 15.
err = 1.0 10^-3

(*Open data file for reading*)
ftext = OpenRead["B:\\experiment\\G\\data\\q\\201308\\20130809\\"<>fstring];

(*Find ref. freq. line in header*)
rmark = Find[ftext, "ref = "];

(*Find position of last byte in this line*)
prtext = StreamPosition[ftext] - 9;

(*Set pointer to this position*)
SetStreamPosition[ftext, prtext];

(*Read reference frequency*)
freqref = ReadList[ftext, Number, 1][[1]];

(*Find beat freq. line in header*)
bmark = Find[ftext, "df = "];

(*Find position of last byte in this line*)
pbtext = StreamPosition[ftext] - 7;

(*Set pointer to this position*)
SetStreamPosition[ftext, pbtext];

(*Read beat frequency*)
freqb = ReadList[ftext, Number, 1][[1]];

(*Find last line in header*)
fmark = Find[ftext, "(V)"];

(*Find position of last byte in this line*)
pftext = StreamPosition[ftext];

(*Set pointer to this position*)
SetStreamPosition[ftext, pftext];

(*Read rest of file as list of ordered number pairs*)
QRawDatF = ReadList[ftext, Number, RecordLists -> True];

(*Close file*)
Close[ftext];

(*Get sign of beat frequency*)
signfreqb = freqb/Abs[freqb]

dimraw = Dimensions[QRawDatF][[1]]

(*select first or second time stamp*)
QRawDat = Table[{QRawDatF[[i,1]],QRawDatF[[i,3]]},{i,1,dimraw}];

(*Plot points*) 
QRawPlt = ListPlot[QRawDat, PlotStyle -> {PointSize[.010]}, 
   PlotRange -> All, PlotLabel -> "Raw Vx vs t"];

(*Show Plots*)
Show[QRawPlt, DisplayFunction-> (Print[#];&)];

(*Separate t data*)
QtRawDat=Table[QRawDat[[n,1]],{n,1,dimraw}];

(*Subtract max and min cuts from data*)
MaxQtRawDatSub = QtRawDat - MaxQtRawDat;
MinQtRawDatSub = QtRawDat - MinQtRawDat;

(*take least integer and find sign; negate where necessary*)
NMaxQtRawDatSub = Sign[Floor[-MaxQtRawDatSub]]; 
NMinQtRawDatSub = Sign[Floor[MinQtRawDatSub]]; 

(*Find positions of negatives - indicates cut elements*)
ZMaxQtRawDatSub = Position[NMaxQtRawDatSub, -1]; 
ZMinQtRawDatSub = Position[NMinQtRawDatSub, -1]; 

(*Find Union*)
UZQRawDatSub = Union[ZMaxQtRawDatSub, ZMinQtRawDatSub];

(*Delete points with empty bins*)
QRawDatCut = Delete[QRawDat, UZQRawDatSub];
QMaxRawDatCut = Delete[QRawDat, ZMaxQtRawDatSub];

(*Calculate size of remaining array*)
DQRaw = Dimensions[QRawDatCut];

(*
(*Re-zero the time data*)
QRawDatCut = Table[{QRawDatCut[[n, 1]] - MinQtRawDat, QRawDatCut[[n, 2]]},{n, DQRaw[[1]]}];
*)

(*Error array*)
Errlist = Table[err, {l,1,DQRaw[[1]]}]

(*Plot points*) 
QRawCutPlt = ListPlot[QRawDatCut, PlotStyle -> {PointSize[.010]}, PlotRange -> All, PlotLabel -> "Vx vs t, with cuts"];

QRawCutLogPlt = ListLogPlot[QRawDatCut, PlotStyle -> {PointSize[.01]}, 
   PlotRange -> All, PlotLabel -> "Vx vs t, with cuts"];

(*Plot with Error Bars*)
QRawDatCutErrPlt =  ErrorListPlot[Table[{{QRawDatCut[[k,1]],QRawDatCut[[k,2]]},ErrorBar[err]},{k,1,DQRaw[[1]]}],PlotRange->All];

(*Show Plots*)
Show[QRawDatCutErrPlt, DisplayFunction -> (Print[#];&)];

(*Fit decaying sinusoid*)
QFits = NonlinearModelFit[QRawDatCut,  
   model, paramlist, ti,
   Weights -> (1./Errlist^2)]

QFit        = QFits[{"BestFitParameters"}][[1]];
QVar        = QFits["EstimatedVariance"];
timecons    = 1./(QFits["ParameterTableEntries"][[2,1]]);
timeconserr = (timecons^2) QFits["ParameterTableEntries"][[2,2]];
frequenc    = QFits["ParameterTableEntries"][[3,1]];
frequencerr = QFits["ParameterTableEntries"][[3,2]];
(*
timecons2    = 0;
timeconserr2 = 0;
timecons2    = 1./(QFits["ParameterTableEntries"][[7,1]]);
timeconserr2 = (timecons^2) QFits["ParameterTableEntries"][[7,2]];
*)

(*Show resulting rule*)
Print[" "]
Print["Final fit Parameters:"]
Print[QFit]
Print[" "]
Print["Chisq/DOF from fit:"]
Print[QVar]
Print[" "]
Print["time const. and error:"]
Print[ScientificForm[timecons], ", ", ScientificForm[timeconserr]]
Print["Beat freq and error:"]
Print[ScientificForm[frequenc], ", ", ScientificForm[frequencerr]]
(*
Print["time const. 2 and error:"]
Print[ScientificForm[timecons2], ", ", ScientificForm[timeconserr2]]
*)
(*Apply rule to function and plot*)

QFitFn = model /. {QFit};

QFitLn = Plot[QFitFn, {ti, MinQtRawDat, MaxQtRawDat}, 
   PlotRange -> All,
   PlotStyle -> {Red}, PlotLabel -> "Fit Function"];

QFitErrLn = Plot[QFitFn, {ti, MinQtRawDat, MaxQtRawDat}, 
   PlotRange -> All,
   PlotStyle -> {Red}, PlotLabel -> "Fit Function"];

QFitLogLn = LogPlot[QFitFn, {ti, MinQtRawDat, MaxQtRawDat}, 
   PlotRange -> All,
   PlotStyle -> {Red}, PlotLabel -> "Fit Function"];

(*Display superposed plots*)
Show[QFitLn, DisplayFunction -> (Print[#];&)];

Print["Fit and Data Superposed:"]
Show[QRawCutPlt, QFitLn, DisplayFunction -> (Print[#];&)];
Show[QRawCutLogPlt, QFitLogLn, DisplayFunction -> (Print[#];&)];
Show[QRawDatCutErrPlt, QFitErrLn, DisplayFunction -> (Print[#];&)];

(*Calculate Residuals*)
QFitRes = Table[{QRawDatCut[[j,1]],QRawDatCut[[j,2]]-(QFitFn /. ti -> QRawDatCut[[j,1]])[[1]]}, {j,1,DQRaw[[1]]}];

(*Calculate square residuals normalized to errors*)
QFitRes2 = Table[{QRawDatCut[[k,1]],(QRawDatCut[[k,2]]-(QFitFn /. ti -> QRawDatCut[[k,1]])[[1]])^2/(err^2)}, {k,1,DQRaw[[1]]}];

(*Chi-sq*)
Chisq = Total[Table[QFitRes2[[m,2]],{m,1,DQRaw[[1]]}]];
ChisqDOF = Chisq/(DQRaw[[1]]-5.);

(*Plot points*) 
QFitResPlt = ListPlot[QFitRes, PlotStyle -> {PointSize[.010]}, PlotRange -> All, PlotLabel -> "Fit Residuals"];
QFitRes2Plt = ListPlot[QFitRes2, PlotStyle -> {PointSize[.010]}, PlotRange -> All, PlotLabel -> "Norm. Sq. Residuals"];

(*Show Plots*)
Show[QFitResPlt, DisplayFunction -> (Print[#];&)];
Show[QFitRes2Plt, DisplayFunction -> (Print[#];&)];
Print["Chisq/DOF (manual) = ",ChisqDOF]

(*Calculate res. freq.*)
resfreqcalc = N[(freqref-(signfreqb freqr)) /. {QFit}][[1]]
Print["res freq from fit = ",NumberForm[resfreqcalc,9]," +/- ",NumberForm[frequencerr,2]]

(*Calculate Q*)
Qcalc = Pi resfreqcalc timecons
Qcalcerr = Pi Sqrt[(resfreqcalc timeconserr)^2 + (timecons frequencerr)^2]
Print["Q from fit = ", Qcalc," +/- ",Qcalcerr]

(*for logbook*)
Print["Q Data, file = ",fstring]
Show[QRawCutLogPlt, QFitLogLn, DisplayFunction -> (Print[#];&)];
Show[QFitResPlt, DisplayFunction -> (Print[#];&)];
Print["Chisq/DOF (manual) = ",ChisqDOF]
Print["res freq from fit = ",NumberForm[resfreqcalc,9]," +/- ",NumberForm[frequencerr,2]]
Print["Q from fit = ", Qcalc," +/- ",Qcalcerr]
