(*Attempt to apply HOM analysis to
noise vs probe bias data [G-IUCEEM-8 logbook p. 64],
uses correct data sets in stat tests*)


Clear[fstring,rn,mlag,p,ofstring,Dimf,dimf,oDimf,odimf,ntext,ontext,rmark,
      ormark,prtext,oprtext,TSSP,oTSSP,FRawDat,oFRawDat,nmark,onmark,
      pntext,opntext,TSSP,oTSSP,VRawDat,oVRawDat,Tclose,oTclose,fv,Nraw,rfv,
      rNraw,pcrate,nl,fftf,xdat,ydat,zdat,mxdat,sxdat,mydat,sydat,xnorm,dhs,
      mzdat,amzdat,vzdat,vzdatc,sdm,sdv,kux,kuy,skx,sky,cdx,hgz,adz,kstx,
      kstxc,ksty,kstyc,adtx,adtxc,adty,adtyc,fftz,afftzs,fftzp,argacv,acv,
      argacf,acf,absacf,arg2acf,absacfp,arg2acfp,sdmc,sdvc,fitlag,gammalo,
      gammahi,gamma,igamma,acvp,alpha,recr,bc,argrt,acfraw,acfARh,absacfARh,
      arg2acfARh,absacfARhp,arg2acfARhp,acffit,sdmcfit,sdvcfit]

(*Enter near res. data files as strings*)
fstring   = {"gdat2013_07_16s002r0001.txt"}

(*Inputs*)
rn   = 5  (*data reduction factor*)
mlag = 100 (*maximum lag for autocorrelation*)
p    = 18   (*auto-regressive model order*)

(*Enter off res. data files as strings*)
ofstring  = {"gdat2013_07_16s002r0001.txt"}

Print["Near Res Data, files:"]
Print[fstring]
Print["Off Res Data, files:"]
Print[ofstring]

(*Get number of files*)
Dimf = Dimensions[fstring]
dimf = Dimf[[1]]
oDimf = Dimensions[ofstring]
odimf = oDimf[[1]]

(*Open data files for reading*)
ntext = Table[OpenRead["B:\\experiment\\G\\data\\201307\\20130717\\"<>fstring[[n]]],
              {n, 1, dimf}]
ontext = Table[OpenRead["B:\\experiment\\G\\data\\201307\\20130717\\"<>ofstring[[n]]],
              {n, 1, odimf}]

(*Find frequency line in header*)
rmark = Table[Find[ntext[[n]], "ds345 freq."], {n,1,dimf}]
ormark = Table[Find[ontext[[n]], "ds345 freq."], {n,1,odimf}]

(*Find position of last byte in this line*)
prtext = Table[StreamPosition[ntext[[n]]]-11, {n,1,dimf}]
oprtext = Table[StreamPosition[ontext[[n]]]-11, {n,1,odimf}]

(*Set pointer to this position*)
TSSP = Table[SetStreamPosition[ntext[[n]], prtext[[n]]], {n,1,dimf}]
oTSSP = Table[SetStreamPosition[ontext[[n]], oprtext[[n]]], {n,1,odimf}]

(*Read rest of file as list of ordered number pairs*)
FRawDat = Table[ReadList[ntext[[n]], Number, 1],
                {n,1,dimf}];
oFRawDat = Table[ReadList[ontext[[n]], Number, 1],
                {n,1,odimf}];

(*Find last line in header*)
nmark = Table[Find[ntext[[n]], "Vx (V)"], {n,1,dimf}]
onmark = Table[Find[ontext[[n]], "Vx (V)"], {n,1,odimf}]

(*Find position of last byte in this line*)
pntext = Table[StreamPosition[ntext[[n]]], {n,1,dimf}]
opntext = Table[StreamPosition[ontext[[n]]], {n,1,odimf}]

(*Set pointer to this position*)
TSSP = Table[SetStreamPosition[ntext[[n]], pntext[[n]]], {n,1,dimf}]
oTSSP = Table[SetStreamPosition[ontext[[n]], opntext[[n]]], {n,1,odimf}]

(*Read rest of file as list of ordered number pairs*)
VRawDat = Table[ReadList[ntext[[n]], Number, RecordLists -> True],
                {n,1,dimf}];
oVRawDat = Table[ReadList[ontext[[n]], Number, RecordLists -> True],
                {n,1,odimf}];

(*Close files*)
Tclose = Table[Close[ntext[[n]]], {n,1,dimf}];
oTclose = Table[Close[ontext[[n]]], {n,1,odimf}];

fv = Flatten[VRawDat,1];

Nraw = Dimensions[fv][[1]];

(*sub-list with each nth element of the data*)

(*
rfv = Table[fv[[rn k -(rn-1)]],{k,1,Nraw/rn}];
*)

rfv = Table[fv[[rn k - rn]],{k,2,Nraw/rn}];

rNraw = Dimensions[rfv][[1]];

pcrate = 10.

(*Nyquist limit for FFT: (PC sample rate/reduction factor)/2*)
nl = pcrate/(2. rn)

(*FFT scaling factor: Nyquist limit/(1/2 total points in data set)*)
fftf = 2. nl/(rNraw)

xdat = Table[rfv[[i,1]],{i,1,rNraw}];

ydat = Table[rfv[[i,2]],{i,1,rNraw}];

zdat = Table[rfv[[i,1]]+I rfv[[i,2]],{i,1,rNraw}];

mxdat = Mean[xdat];

sxdat = StandardDeviation[xdat];

mydat = Mean[ydat];

sydat = StandardDeviation[ydat];

xnorm = (xdat-mxdat)/sxdat;

dhs = HistogramDistribution[xnorm];

mzdat = Mean[zdat];

amzdat = Abs[mzdat];

vzdat = Variance[zdat]

vzdatc = sxdat^2+sydat^2

sdm = Sqrt[vzdat/rNraw]

(*
sdv = vzdat Sqrt[2/rNraw]
*)

sdv = Sqrt[vzdat^2/rNraw]

kux = Kurtosis[xdat]-3.;

kuy = Kurtosis[ydat]-3.;

skx = Skewness[xdat];

sky = Skewness[ydat];

cdx = Plot[CDF[dhs,i],{i,-4,4}];

hgz = Histogram[Table[Arg[zdat[[i]]],{i,1,rNraw}],
      AxesLabel->{"arg(z)","samples"}];

adz = ListPlot[Table[{Re[zdat[[i]]],Im[zdat[[i]]]},{i,1,rNraw}],
      Frame->True,AxesLabel->{"Im(z)","Re(z)"}];

Show[cdx, DisplayFunction -> (Print[#];&)]
Show[hgz, DisplayFunction -> (Print[#];&)]
Show[adz, DisplayFunction -> (Print[#];&)]

Print["raw data N = ",Nraw]
Print["reduced data N = ",rNraw]
Print["Nyquist limit, Hz = ",nl]
Print["abs mean z = ",amzdat]
Print["variance z = ",vzdat]
Print["variance z from x,y = ",vzdatc]
Print["std dev mean = ",sdm]
Print["std dev var = ",sdv]
Print["x kurtosis = ",kux]
Print["y kurtosis = ",kuy]
Print["x skewness = ",skx]
Print["y skewness = ",sky]
(*
kstx = KolmogorovSmirnovTest[xdat,NormalDistribution[mxdat,sxdat],"TestDataTable",Method->"MonteCarlo"];

kstxc = KolmogorovSmirnovTest[xdat,NormalDistribution[mxdat,sxdat],"TestConclusion"];

ksty = KolmogorovSmirnovTest[ydat,NormalDistribution[mydat,sydat],"TestDataTable"];

kstyc = KolmogorovSmirnovTest[ydat,NormalDistribution[mydat,sydat],"TestConclusion"];

Print["KS test, x: ",kstx]
Print[kstxc]
Print["KS test, y: ",ksty]
Print[kstyc]
*)
adtx = AndersonDarlingTest[xdat,NormalDistribution[mxdat,sxdat],"TestDataTable"];

adtxc = AndersonDarlingTest[xdat,NormalDistribution[mxdat,sxdat],"TestConclusion"];

adty = AndersonDarlingTest[ydat,NormalDistribution[mydat,sydat],"TestDataTable"];

adtyc = AndersonDarlingTest[ydat,NormalDistribution[mydat,sydat],"TestConclusion"];

Print["AD test, x: ",adtx]
Print[adtxc]
Print["AD test, y: ",adty]
Print[adtyc]

(*FFT*)
fftz = Fourier[zdat];

(*scale real FFT*)
afftzs = Table[{fftf(i-1),Abs[fftz[[i]]]},{i,1,rNraw/2}];

(*plot real FFT*)
fftzp = ListLinePlot[afftzs,PlotRange->All,AxesLabel->{"Freq(Hz)","amp"}];

Show[fftzp, DisplayFunction -> (Print[#];&)]

(*autocorrelation*)

(*Table of rho(h) elements, one value of h per row*)
argacv = Table[(1/(rNraw-lag))(zdat[[i]] Conjugate[zdat[[i+lag]]]-amzdat^2),{lag,0,mlag},{i,1,rNraw-lag}];

(*sum each row, get 1D array of rho(h)s*)
acv = Total[argacv,{2}];

argacf = Table[(1/vzdat)(1/(rNraw-lag))(zdat[[i]] Conjugate[zdat[[i+lag]]]-amzdat^2),{lag,0,mlag},{i,1,rNraw-lag}];

(*sum each row, get 1D array of rho(h)s*)
acf = Total[argacf,{2}];

(*plot arrays*)
absacf=Table[{m,Abs[acf[[m+1]]]},{m,0,mlag-1}];

arg2acf=Table[{m,(Arg[acf[[m+1]]]/(2. Pi))^2},{m,0,mlag-1}];

absacfp = ListPlot[absacf,PlotRange->All,AxesLabel->{"lag","acf"},AxesOrigin->{0,0}];

arg2acfp = ListPlot[arg2acf,PlotRange->All,AxesLabel->{"lag","arg(acf)"},AxesOrigin->{0,0}];

(*show plots*)
Show[absacfp, DisplayFunction -> (Print[#];&)]

Show[arg2acfp, DisplayFunction -> (Print[#];&)]

(*corrected variance of sample mean*)
sdmc=(vzdat/rNraw) (1.+2. Total[Table[(1-k/rNraw) Re[acf[[k+1]]],{k,1,mlag-1}]]);

(*corrected variance of sample variance*)
sdvc=(vzdat^2/rNraw) (1.+2. Total[Table[(1-k/rNraw) Abs[acf[[k+1]]]^2,{k,1,mlag-1}]]);

Print["corr std dev mean = ",Sqrt[sdmc]]
Print["corr std dev var = ",Sqrt[sdvc]]

(*auto-regressive model*)
Print["----- AR Model -----"]
fitlag = mlag-p (*number of points to generate*)

Print["order = ",p]
Print["First ",p," acv values (gamma vector), starting with acv(1)"]
Print[MatrixForm[Table[acv[[i]],{i,2,p+1}]]]

(*gamma matrix*)
gammalo = SparseArray[Table[Band[{i,1}]->acv[[i]],{i,1,p}],{p,p}];
gammahi = SparseArray[Table[Band[{1,i}]->Conjugate[acv[[i]]],{i,2,p}],{p,p}];
gamma = gammalo+gammahi

Print["AR Model gamma matrix"]
Print[MatrixForm[gamma]]

(*Yule-Walker eq.*)
igamma = Inverse[gamma];
acvp = Take[acv,{2,p+1}];
alpha = Dot[igamma,acvp];

Print["Inverse gamma matrix"]
Print[MatrixForm[igamma]]

Print["AR Model fit parameters to minimize residuals (alpha vector):"]
Print[MatrixForm[alpha]]

(*recursion relation*)
recr = acfAR[n] == Dot[alpha,Table[acfAR[n-i],{i,1,p}]]; (*symbolic eq,HOM 21*)
bc   = Table[acfAR[i]==acf[[i]],{i,1,p}]; (*initial conditions*)
argrt = Prepend[bc,recr]; (*argument of recurrence table*)

acfraw = Table[acf[[i]],{i,1,p}]
Print["First ",p," acf values (rho vector), starting with rho(0)"]
Print[MatrixForm[acfraw]]

Print["recurrence table syntax"]
Print[argrt]

acfARh = RecurrenceTable[argrt,acfAR,{n,p+1,p+fitlag}]; (*AR model
acfs*)

Print["next ",fitlag," acf values from AR model"]
Print[acfARh]

(*plot arrays*)
absacfARh=Table[{m+p,Abs[acfARh[[m+1]]]},{m,0,fitlag-1}];

arg2acfARh=Table[{m+p,(Arg[acfARh[[m+1]]]/(2. Pi))^2},{m,0,fitlag-1}];

absacfARhp = ListLinePlot[absacfARh,PlotRange->All,PlotLabel->"acf
with AR fit",AxesLabel->{"lag","acf"},AxesOrigin->{0,0}];

arg2acfARhp = ListLinePlot[arg2acfARh,PlotRange->All,AxesLabel->{"lag","arg(acf)"},AxesOrigin->{0,0}];

(*show plots*)
Show[absacfp,absacfARhp, DisplayFunction -> (Print[#];&)]

(*re-calculate varainces*)
acffit = Join[acfraw,acfARh];
sdmcfit=(vzdat/rNraw) (1.+2. Total[Table[(1-k/rNraw) Re[acffit[[k+1]]],{k,1,mlag-1}]]);
sdvcfit=(vzdat^2/rNraw) (1.+2. Total[Table[(1-k/rNraw) Abs[acffit[[k+1]]]^2,{k,1,mlag-1}]]);

Print["----------------------"]
Print["Result for plot/fitting:"]
Print["variance z = ",vzdat]
Print["corr std dev var     = ",Sqrt[sdvc]]
Print["corr std dev var fit = ",Sqrt[sdvcfit]]
Print["std dev z = ",Sqrt[vzdat]]
Print["corr std dev var     = ",Sqrt[sdvc]/(2 Sqrt[vzdat])]
Print["corr std dev var fit = ",Sqrt[sdvcfit]/(2 Sqrt[vzdat])]


